/*
 *     __   ___   _
 *    /  \ |   \ | | /\---------<ShellCode Generator Library>----------------->
 *   / /\ \| |\ \| |/ /
 *  / /__\ \    /|   /          libShellCode is an ORK's project!
 *  \______/ |\ \| | \          For comments, bug reports, etc ...
 *<--------|_| \_\_|\_\         contact me at orkmail<at>katamail<dot>com
 *
 *
 */


#include <stdio.h>
#include <stdlib.h>

// Find the actual memory adress
char *JMP;
char *CALLPOP;
char *POP_ESI;

// Zeroing a register
char *XOR_EAX_EAX;
char *XOR_EBX_EBX;
char *XOR_ECX_ECX;
char *XOR_EDX_EDX;

// Move a value to a register
char *MOV_IN_AL;
char *MOV_IN_CL;
char *MOV_IN_DL;
char *MOV_IN_CH;
char *MOV_IN_CX;
char *MOV_IN_ECX;

// Move a register to a register
char *MOV_EAX_EBX;
char *MOV_EAX_ECX;
char *MOV_ECX_EAX;
char *MOV_ESI_EBX;
char *MOV_ESI_ECX;
char *MOV_EDI_EBX;
char *XCHG_EAX_EDI;

// Move a register to memory
char *MOV_DL_nESI; 
char *MOV_AL_nESI; 
char *MOV_ESI_nESI;
char *MOV_EAX_nESI;
char *MOV_EBX_nESI;

// Increment, decrement a register
char *INC_EAX;
char *INC_EBX;

// Add and sub
char *ADD_BL;
char *ADD_ECX_ESI;
char *SUB_EBX;

// XOR  memory
char *XOR_AL_EBX;

// XOR  a register
char *XOR_ECX;

// Lea
char *LEA_nEBX_ECX;
char *LEA_nESI_ECX; 
char *LEA_nESI_EDX;
char *LEA_nESI_ESI;

// Loop
char *LOOP; 
char *CMP_EAX_EDX;
char *CMP_EAX_EBX;
char *JNE;
char *CMP_CX_nESP;
char *CMP_CX_nEBP;
char *CMP_ECX_nESP;
char *CMP_ECX_nEBP;
char *JNE;

// Interrupt call
char *INT_80;

// Push
char *PUSH_EAX;
char *PUSH_ECX;
char *PUSH_EDX;
char *PUSH_ESI;
char *PUSH_CX;
char *PUSH;

// NOPs

char *NOP;


/*
 * Function:    initialize_i386_codes()
 * Parameters:
 * Return:      
 * Description: Initialize the codes used to build the ShellCodes on i386 architecture
 *
 */


void initialize_i386_codes()
{
  // Find the actual memory adress
  JMP          = (char *) strdup("\xeb\x01");               // jmp    call
  CALLPOP      = (char *) strdup("\xe8\x01\xff\xff\xff");   // call   pop
  POP_ESI      = (char *) strdup("\x5e");                   // popl   %esi
  
  // zeroing a register
  XOR_EAX_EAX  = (char *) strdup("\x31\xc0");               // xor    %eax, %eax
  XOR_EBX_EBX  = (char *) strdup("\x31\xdb");               // xor    %ebx, %ebx
  XOR_ECX_ECX  = (char *) strdup("\x31\xc9");               // xor    %ecx, %ecx
  XOR_EDX_EDX  = (char *) strdup("\x31\xd2");               // xor    %edx, %edx 
  
  // Move a value to a register
  MOV_IN_AL    = (char *) strdup("\xb0\x01");               // movb   $0x1, %al
  MOV_IN_CL    = (char *) strdup("\xb1\x01");               // movb   $0x1, %cl
  MOV_IN_DL    = (char *) strdup("\xb2\x01");               // movb   $0x1, %dl
  MOV_IN_CH    = (char *) strdup("\xb5\x01");               // movb   $0x1, %ch
  MOV_IN_CX    = (char *) strdup("\x66\xb9\x01\x01");       // movw   $0x101, %cx
  MOV_IN_ECX   = (char *) strdup("\xb9\x01\x01\x01\x01");   // mov    $0x01010101,%ecx

  // Move a register to a register
  MOV_EAX_EBX  = (char *) strdup("\x89\xc3");               // mov     %eax,%ebx
  MOV_EAX_ECX  = (char *) strdup("\x89\xc1");               // mov     %eax,%ecx  
  MOV_ECX_EAX  = (char *) strdup("\x89\xc8");               // mov     %ecx,%eax  
  MOV_ESI_EBX  = (char *) strdup("\x89\xf3");               // mov     %esi,%ebx
  MOV_ESI_ECX  = (char *) strdup("\x89\xf1");               // mov     %esi,%ecx
  MOV_EDI_EBX  = (char *) strdup("\x89\xfb");               // mov     %edi,%ebx 
  XCHG_EAX_EDI = (char *) strdup("\x97");                   // xchg   %eax,%edi

  // Move a register to memory
  MOV_DL_nESI  = (char *) strdup("\x88\x56\x01");           // movb    %dl, 0x1(%esi)    
  MOV_AL_nESI  = (char *) strdup("\x88\x46\x01");           // movb    %al, 0x1(%esi)       
  MOV_ESI_nESI = (char *) strdup("\x89\x76\x01");           // mov     %esi,0x1(%esi)
  MOV_EAX_nESI = (char *) strdup("\x89\x46\x01");           // mov     %eax,0x1(%esi)
  MOV_EBX_nESI = (char *) strdup("\x89\x5e\x01");           // mov     %ebx,0x1(%esi)  
  
  // Increment, decrement a register
  INC_EAX      = (char *) strdup("\x40");                   // inc    %eax 
  INC_EBX      = (char *) strdup("\x43");                   // inc    %ebx 

  // Add and sub
  ADD_BL       = (char *) strdup("\x80\xc3\x01");           // add    $0x1, %bl
  ADD_ECX_ESI  = (char *) strdup("\x01\xce");               // add    %ecx,%esi  
  SUB_EBX      = (char *) strdup("\x83\xeb\x01");           // sub    $0x1, %ebx

  // XOR memory
  XOR_AL_EBX   = (char *) strdup("\x30\x03");               // xor    %al,  (%ebx)

  // XOR  a register
  XOR_ECX      = (char *) strdup("\x81\xf1\x01\x01\x01\x01"); // xor    0x1010101,%ecx

  // Lea
  LEA_nEBX_ECX = (char *) strdup("\x8d\x4b\x01");           // leal   0x1(%ebx), %ecx 
  LEA_nESI_ECX = (char *) strdup("\x8d\x4e\x01");           // leal   0x1(%esi), %ecx 
  LEA_nESI_EDX = (char *) strdup("\x8d\x56\x01");           // leal   0x1(%esi), %edx 
  LEA_nESI_ESI = (char *) strdup("\x8d\x76\x01");           // leal   0x1(%esi), %esi

  // Loop and jumps
  LOOP         = (char *) strdup("\xe2\xfb");               // loop   xor
  CMP_EAX_EDX  = (char *) strdup("\x39\xc2");               // cmp    %eax, %edx 
  CMP_EAX_EBX  = (char *) strdup("\x39\xc3");               // cmp    %eax, %ebx 
  JNE          = (char *) strdup("\x75\x01");               // je                
  CMP_CX_nESP  = (char *) strdup("\x66\x39\x4c\x24\x01");   // cmp    %cx,0xe(%esp,1)
  CMP_CX_nEBP  = (char *) strdup("\x66\x39\x4d\x01");       // cmp    %cx,0x1(%ebp)
  CMP_ECX_nESP = (char *) strdup("\x39\x4c\x24\x01");       // cmp    %ecx,0x10(%esp,1)
  CMP_ECX_nEBP = (char *) strdup("\x39\x4d\x01");           // cmp    %ecx,0x1(%ebp)

  // Interrupt calling
  INT_80       = (char *) strdup("\xcd\x80");               // int    $0x80

  // Push
  PUSH_EAX     = (char *) strdup("\x50");                   // push   %eax
  PUSH_ECX     = (char *) strdup("\x51");                   // push   %ecx
  PUSH_EDX     = (char *) strdup("\x52");                   // push   %edx
  PUSH_ESI     = (char *) strdup("\x56");                   // push   %esi
  PUSH_CX      = (char *) strdup("\x66\x51");               // push   %cx
  PUSH         = (char *) strdup("\x68\x01\x01\x01\x01");   // push   $0x01010101

  //NOPs
  NOP          = (char *) strdup("\x27"                     // daa         '
                                 "\x2f"                     // das         /
				 "\x37"                     // aaa         7
				 "\x3f"                     // aas         ?
				 "\x40"                     // inc %eax    @
				 "\x41"                     // inc %ecx    A
				 "\x42"                     // inc %edx    B
				 "\x43"                     // inc %ebx    C
				 "\x44"                     // inc %esp    D
				 "\x45"                     // inc %ebp    E
				 "\x46"                     // inc %esi    F
				 "\x47"                     // inc %edi    G
				 "\x48"                     // dec %eax,   H
				 "\x4a"                     // dec %edx    J
				 "\x4b"                     // dec %ebx    K
				 "\x4c"                     // dec %esp    L
				 "\x4d"                     // dec %ebp,   M
				 "\x4e"                     // dec %esi    N
				 "\x4f"                     // dec %edi    O
				 "\x50"                     // push %eax   P
				 "\x51"                     // push %ecx   Q
				 "\x52"                     // push %edx   R
				 "\x53"                     // push %ebx   S
				 "\x54"                     // push %dsp   T
				 "\x55"                     // push %ebp   U
				 "\x56"                     // push %esi   V
				 "\x57"                     // push %edi   W
				 "\x58"                     // pop %eax    X
				 "\x59"                     // pop %ecx    Y
				 "\x5a"                     // pop %edx    Z
				 "\x5b"                     // pop %ebx    [
				 "\x5d"                     // pop %ebp    ]
				 "\x5e"                     // pop %esi    ^
				 "\x5f"                     // pop %edi    _
				 "\x60"                     // pusha       `
				 "\x90"                     // regular NOP
				 "\x91"                     // xchg %eax,%ecx
				 "\x92"                     // xchg %eax,%edx
				 "\x93"                     // xchg %eax,%ebx
				 "\x95"                     // xchg %eax,%ebp
				 "\x96"                     // xchg %eax,%esi
				 "\x97"                     // xchg %eax,%edi
				 "\x98"                     // cwtl
				 "\x99"                     // cltd
				 "\x9b"                     // fwait
				 "\x9c"                     // pushf
				 "\x9e"                     // safh
				 "\x9f"                     // lahf
				 "\xf5"                     // cmc
				 "\xf8"                     // clc
				 "\xf9"                     // stc
				 "\xfc"                     // cld
				 "\x00");                // Terminate the list
}


/*
 * Function:    free_i386_codes()
 * Parameters:
 * Return:
 * Description: Free the memory used for the i386 codes
 *
 */

void free_i386_codes()
{
  free(JMP);
  free(CALLPOP);     
  free(POP_ESI);
   
  free(XOR_EAX_EAX);  
  free(XOR_EBX_EBX);  
  free(XOR_ECX_ECX);  
  free(XOR_EDX_EDX);

  free(MOV_IN_AL);  
  free(MOV_IN_CL); 
  free(MOV_IN_DL);   
  free(MOV_IN_CH);
  free(MOV_IN_CX);  
  free(MOV_IN_ECX);

  free(MOV_EAX_EBX); 
  free(MOV_EAX_ECX);
  free(MOV_ECX_EAX);
  free(MOV_ESI_EBX);
  free(MOV_ESI_ECX);
  free(MOV_EDI_EBX);
  free(XCHG_EAX_EDI);

  free(MOV_DL_nESI); 
  free(MOV_AL_nESI);  
  free(MOV_ESI_nESI);
  free(MOV_EAX_nESI);    
  free(MOV_EBX_nESI);

  free(INC_EAX);   
  free(INC_EBX); 

  free(ADD_BL); 
  free(ADD_ECX_ESI);
  free(SUB_EBX);

  free(XOR_AL_EBX);

  free(XOR_ECX);

  free(LEA_nEBX_ECX); 
  free(LEA_nESI_ECX); 
  free(LEA_nESI_EDX);
  free(LEA_nESI_ESI);

  free(LOOP);
  free(CMP_EAX_EDX);
  free(CMP_EAX_EBX);
  free(JNE);
  free(CMP_CX_nESP);
  free(CMP_CX_nEBP);
  free(CMP_ECX_nESP);
  free(CMP_ECX_nEBP);

  free(INT_80); 

  free(PUSH_EAX);
  free(PUSH_ECX);
  free(PUSH_EDX);
  free(PUSH_ESI);
  free(PUSH_CX);
  free(PUSH);

  free(NOP);
}
