/*
 *     __   ___   _
 *    /  \ |   \ | | /\---------<ShellCode Generator Library>----------------->
 *   / /\ \| |\ \| |/ /
 *  / /__\ \    /|   /          libShellCode is an ORK's project!
 *  \______/ |\ \| | \          For comments, bug reports, etc ...
 *<--------|_| \_\_|\_\         contact me at orkmail<at>katamail<dot>com
 *
 *
 */


#include <stdio.h>
#include <ncurses.h>
#include <sys/ioctl.h>
#include "scbuilder.h"


#define SCBUILDER_VERSION "0.1.2"

#define MAIN_COLOR 1
#define MENU_COLOR 2
#define HELP_LINE_COLOR 3
#define POPUP_MSG_COLOR 4
#define ERROR_MSG_COLOR 5

#define LEN_CMD 256  // max length of input string in read_str() function
#define LENROW 76    // max length of a ShellCode description line 

int num_arch;
int SCREEN_ROW, SCREEN_COL;
WINDOW *title_win, *arch_win, *os_win, *sc_win, *opt_win, *help_win, *back_win, *back2_win;

// Prototipes
int arch_loop();
int os_loop(int arch_selected);
int sc_loop(int arch_selected, int os_selected);
int options_loop(int arch_selected, int os_selected, int sc_selected);
int select_opt(submenu *sub, char* title);
void read_str(char *string, char *title);
void refresh_back();
void refresh_arch(int item_selected);
void refresh_os(int arch_selected, int os_selected);
void refresh_sc(int arch_selected, int os_selected, int sc_selected);
void refresh_opt(int arch_selected, int os_selected, int sc_selected, int opt_selected);
void refresh_sel_opt(WINDOW *win, submenu *sub, int num_intems, int selected);
void set_default(int arch_selected, int os_selected, int sc_selected);
void clean_help_line(int line);
char *make_SC_printable(char *code);
void refresh_output(WINDOW *win, int selected, char *printable_sc, int real_sc_len);
char *make_desc(char *string);
void build_sc(architecures arch, os system, shellcodes sc);
void about();
void error_msg(char *message);


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

main()
{
  int c, h, i;
  struct winsize  ws = {0, 0, 0, 0};

  // Read the size of the screen 
  ioctl(0, TIOCGWINSZ, &ws);

  SCREEN_ROW = ws.ws_row;
  SCREEN_COL = ws.ws_col;

  if ( (SCREEN_ROW < 24) || (SCREEN_COL < 80) )
    {
      printf("Screen must be at least 25x80 !!\n\n");
      exit(0);
    }

  // initialize ncurses 
  initscr();
  noecho();
  cbreak();      // Line buffering disabled. pass on everything
  curs_set(0);   // Cursor invisible
  start_color(); // Activate the colours

  // Set the Colors
  init_pair(MAIN_COLOR, COLOR_CYAN, COLOR_BLUE);  
  init_pair(MENU_COLOR, COLOR_BLACK, COLOR_WHITE);
  init_pair(POPUP_MSG_COLOR, COLOR_BLACK, COLOR_WHITE);
  init_pair(ERROR_MSG_COLOR, COLOR_RED, COLOR_WHITE);
  init_pair(HELP_LINE_COLOR, COLOR_WHITE, COLOR_BLUE);

  // Count how many architectures, is supported
  num_arch = 0;
  while(arch[num_arch].ID != NULL) num_arch++;

  // Crete the title Bar windows
  title_win = newwin(1, SCREEN_COL, 0, 0);
  wbkgdset(title_win, COLOR_PAIR(MENU_COLOR));
  werase(title_win);
  mvwprintw(title_win, 0, 2, "ShellCode Builder %s", SCBUILDER_VERSION);
  mvwprintw(title_win, 0, SCREEN_COL - 8, "By ORK");
  wrefresh(title_win);

  // Create arch window
  arch_win = newwin(12, 14, 1, 0);
  keypad(arch_win, TRUE); // Activate all keys 
  wbkgdset(arch_win, COLOR_PAIR(MAIN_COLOR));
  werase(arch_win);
  box(arch_win, 0, 0);
  mvwprintw(arch_win, 0, 1, "Architecture");
  wrefresh(arch_win);

  // Create os window
  os_win = newwin(12, 10, 1, 14);
  keypad(os_win, TRUE); // Activate all keys 
  wbkgdset(os_win, COLOR_PAIR(MAIN_COLOR));
  werase(os_win);
  box(os_win, 0, 0);
  mvwprintw(os_win, 0, 4, "OS");
  wrefresh(os_win);

  // Create shellcodes window
  sc_win = newwin(12, 15, 1, 24);
  keypad(sc_win, TRUE); // Activate all keys 
  wbkgdset(sc_win, COLOR_PAIR(MAIN_COLOR));
  werase(sc_win);
  box(sc_win, 0, 0);
  mvwprintw(sc_win, 0, 3, "ShellCode");
  wrefresh(sc_win);

  // Create options window
  opt_win = newwin(12, SCREEN_COL - 39, 1, 39);
  keypad(opt_win, TRUE); // Activate all keys 
  wbkgdset(opt_win, COLOR_PAIR(MAIN_COLOR));
  werase(opt_win);
  box(opt_win, 0, 0);
  mvwprintw(opt_win, 0, 2, "Options");
  wrefresh(opt_win);

  // Crete Help windows
  help_win = newwin(6, SCREEN_COL , 13, 0);
  wbkgdset(help_win, COLOR_PAIR(HELP_LINE_COLOR));
  werase(help_win);
  box(help_win, 0, 0);
  wrefresh(help_win);

  // Crete back windows
  back_win = newwin(SCREEN_ROW - 12 - 6 - 1, SCREEN_COL , 12 + 6 + 1, 0);
  wbkgdset(back_win, COLOR_PAIR(HELP_LINE_COLOR));
  refresh_back();

  arch_loop();
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

int arch_loop()
{
  int c, actual;
  actual = 0;
  while(1)
    {
      refresh_arch(actual);
      c = wgetch(arch_win);
      switch(c)
        {

	case KEY_UP:
	  if (actual != 0) actual --;
	  break;

	case KEY_DOWN:
	  if (actual != num_arch - 1) actual ++;
	  break;

	case 10: // ENTER
	  os_loop(actual);
	  break;

	case 'q':
	case 'Q':
	  endwin();
	  exit(0);
	  break;
	}
 
    }
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

int os_loop(int arch_selected)
{
  int c, actual;
  actual = 0;
  while(1)
    {
      refresh_os(arch_selected, actual);
      c = wgetch(arch_win);
      switch(c)
        {

	case KEY_UP:
	  if (actual != 0) actual --;
	  break;

	case KEY_DOWN:
	  if (actual != arch[arch_selected].num_os - 1) actual ++;
	  break;
	  
	case 10: // ENTER
	  sc_loop(arch_selected, actual);
	  break;
 
	case 127:        // BACKSPACE on BSD
	case KEY_BACKSPACE:
	  clean_help_line(2);
	  werase(os_win);
	  box(os_win, 0, 0);
	  mvwprintw(os_win, 0, 4, "OS");
	  wrefresh(os_win);
	  return;
	  break;

	case 'q':
	case 'Q':
	  endwin();
	  exit(0);
	  break;
	}
    }
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

int sc_loop(int arch_selected, int os_selected)
{
  int c, actual;
  actual = 0;
  while(1)
    {
      refresh_sc(arch_selected, os_selected, actual);
      c = wgetch(arch_win);
      switch(c)
        {

	case KEY_UP:
	  if (actual != 0) actual --;
	  break;

	case KEY_DOWN:
	  if (actual != arch[arch_selected].systems[os_selected].num_sc - 1) actual ++;
	  break;
	  
	case 10: // ENTER
	  set_default(arch_selected, os_selected, actual);
	  options_loop(arch_selected, os_selected, actual);
	  break;

	case 127:        // BACKSPACE on BSD
	case KEY_BACKSPACE:
	  clean_help_line(3);
	  werase(sc_win);
	  box(sc_win, 0, 0);
	  mvwprintw(sc_win, 0, 3, "ShellCode");
	  wrefresh(sc_win);
	  return;
	  break;

	case 'q':
	case 'Q':
	  endwin();
	  exit(0);
	  break;
	}
    }
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

int options_loop(int arch_selected, int os_selected, int sc_selected)
{
  int c, actual, r;
  actual = 0;
  while(1)
    {
      refresh_opt(arch_selected, os_selected, sc_selected, actual);
      c = wgetch(arch_win);
      switch(c)
        {

	case KEY_UP:
	  if (actual != 0) actual --;
	  break;

	case KEY_DOWN:
	  if (actual != arch[arch_selected].systems[os_selected].sc[sc_selected].num_opt - 1) actual ++;
	  break;
	  
	case 10: // ENTER

	  if (arch[arch_selected].systems[os_selected].sc[sc_selected].func[actual].type == 0)
	  {
	    r = select_opt(arch[arch_selected].systems[os_selected].sc[sc_selected].func[actual].sub, arch[arch_selected].systems[os_selected].sc[sc_selected].func[actual].ID);
	    arch[arch_selected].systems[os_selected].sc[sc_selected].func[actual].valI = r;
	  }
	      else
	  {
	    read_str(arch[arch_selected].systems[os_selected].sc[sc_selected].func[actual].valC, arch[arch_selected].systems[os_selected].sc[sc_selected].func[actual].ID);
	  }

	  werase(help_win);
	  box(help_win, 0, 0);
	  refresh_arch(arch_selected);
	  refresh_os(arch_selected, os_selected);
	  refresh_sc(arch_selected, os_selected, sc_selected);
	  refresh_opt(arch_selected, os_selected, sc_selected, actual);
	  refresh_back();
	  break;

	case KEY_F(5):
	  build_sc(arch[arch_selected], arch[arch_selected].systems[os_selected], arch[arch_selected].systems[os_selected].sc[sc_selected]);
	  werase(help_win);
	  box(help_win, 0, 0);
	  refresh_arch(arch_selected);
	  refresh_os(arch_selected, os_selected);
	  refresh_sc(arch_selected, os_selected, sc_selected);
	  refresh_opt(arch_selected, os_selected, sc_selected, actual);
	  refresh_back();
	  break;

	case 127:        // BACKSPACE on BSD
	case KEY_BACKSPACE:
	  clean_help_line(4);
	  werase(opt_win);
	  box(opt_win, 0, 0);
	  mvwprintw(opt_win, 0, 2, "Options");
	  wrefresh(opt_win);
	  free(arch[arch_selected].systems[os_selected].sc[sc_selected].func[actual].valC);
	  return;
	  break;

	case 'a':
	case 'A':
	  about();
	  werase(help_win);
	  box(help_win, 0, 0);
	  refresh_arch(arch_selected);
	  refresh_os(arch_selected, os_selected);
	  refresh_sc(arch_selected, os_selected, sc_selected);
	  refresh_opt(arch_selected, os_selected, sc_selected, actual);
	  refresh_back();
	  break;

	case 'q':
	case 'Q':
	  endwin();
	  exit(0);
	  break;
	}
    }
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

int select_opt(submenu *sub, char *title)
{
  int max_len, num_opt=0, win_len, selected=0, c, h;
  WINDOW *opt_win;

  max_len = strlen(title);
  while (sub[num_opt].desc != NULL)
    {
      if (max_len < strlen(sub[num_opt].desc)) max_len = strlen(sub[num_opt].desc);
      num_opt++;
    }

  win_len = ((SCREEN_COL - (max_len + 4))/2);
  
  // Create error window
  opt_win = newwin(num_opt + 4, max_len + 4, ((SCREEN_ROW - (num_opt+4))/2), win_len);
  keypad(opt_win, TRUE); // Activate all keys 
  wbkgdset(opt_win, COLOR_PAIR(POPUP_MSG_COLOR));
  werase(opt_win);
  box(opt_win, 0, 0);

  mvwprintw(opt_win, 1, 2, "%s", title);

  while(1)
    {
      refresh_sel_opt(opt_win, sub, num_opt, selected);
      c = wgetch(arch_win);
      switch(c)
        {

	case KEY_UP:
	  if (selected != 0) selected --;
	  refresh_sel_opt(opt_win, sub, num_opt, selected);
	  break;

	case KEY_DOWN:
	  if (selected != num_opt-1) selected ++;
	  refresh_sel_opt(opt_win, sub, num_opt, selected);
	  break;
	  
	case 10: // ENTER
	  delwin(opt_win);
	  return(sub[selected].val);
	  break;
	}
    }

  wrefresh(opt_win);
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void read_str(char *string, char *title)
{
  int win_len, max_len, c, len;
  char *pt;
  WINDOW *str_win;
  curs_set(1);
  max_len = 22;
  win_len = ((SCREEN_COL - (max_len + 4))/2);
  
  // Create input window
  str_win = newwin(6, max_len + 4, ((SCREEN_ROW - 4)/2), win_len);
  keypad(str_win, TRUE); // Activate all keys 
  wbkgdset(str_win, COLOR_PAIR(POPUP_MSG_COLOR));
  werase(str_win);
  box(str_win, 0, 0);
  mvwprintw(str_win, 1, 2, "%s", title);
  mvwprintw(str_win, 3, 2, ":");
  wrefresh(str_win);

  len = strlen(string);
  pt = string;
  if (len > 20)  pt = pt + len - 20;
  mvwprintw(str_win, 3, 4, "%s", pt);
  wrefresh(str_win); 
      
  while(1)
    {
      c = wgetch(arch_win);
     
      if(isprint(c))
      {
	if (len < LEN_CMD)
	  {
	    len++;
	    sprintf(string, "%s%c", string, c);
	  }
      }

      if (c == KEY_BACKSPACE || c == 127)
	{
	  if(len > 0)
	    {
	      len --;
	      string[len] = 0;
	    }
	}

      if (c == 10) // Enrer
	{
	  curs_set(0);
	  return;
	}     
      mvwprintw(str_win, 3, 4, "                    ");
      pt = string;
      if (len > 20)  pt = pt + len - 20;

      mvwprintw(str_win, 3, 4, "%s", pt);
      wrefresh(str_win); 
    }
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void refresh_back()
{
  werase(back_win);
  mvwprintw(back_win, 1, 1, "ENTER = Select        F5 = Build Shellcode");
  mvwprintw(back_win, 2, 1, "BACKSPACE = Back      Q = Quit");
  mvwprintw(back_win, 3, 1, "ARROWS = Up and Down  A = About");
  wrefresh(back_win);
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void refresh_arch(int item_selected)
{
  int i;

  werase(arch_win);
  box(arch_win, 0, 0);
  mvwprintw(arch_win, 0, 1, "Architecture");

  for(i = 0; i < num_arch; i++)
    {
      if (i == item_selected)
	{
	  wattron(arch_win, A_REVERSE); 
	  mvwprintw(arch_win, i+1, 1, arch[i].ID);
	  wattroff(arch_win, A_REVERSE); 
	}
      else
	{
	  mvwprintw(arch_win, i+1, 1, arch[i].ID);
	}
    }
  wrefresh(arch_win);
  clean_help_line(1);
  mvwprintw(help_win, 1, 2, arch[item_selected].desc);
  wrefresh(help_win);
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void refresh_os(int arch_selected, int os_selected)
{
  int i;

  werase(os_win);
  box(os_win, 0, 0);
  mvwprintw(os_win, 0, 4, "OS");

  for(i = 0; i < arch[arch_selected].num_os; i++)
    {
      if (i == os_selected)
	{
	  wattron(os_win, A_REVERSE); 
	  mvwprintw(os_win, i+1, 1, arch[arch_selected].systems[i].ID);
	  wattroff(os_win, A_REVERSE); 
	}
      else
	{
	  mvwprintw(os_win, i+1, 1, arch[arch_selected].systems[i].ID);
	}
    }
  wrefresh(os_win);
  clean_help_line(2);
  mvwprintw(help_win, 2, 2, arch[arch_selected].systems[os_selected].desc);
  wrefresh(help_win);
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void refresh_sc(int arch_selected, int os_selected, int sc_selected)
{
  int i;
  
  werase(sc_win);
  box(sc_win, 0, 0);
  mvwprintw(sc_win, 0, 3, "ShellCode");

  for(i = 0; i < arch[arch_selected].systems[os_selected].num_sc; i++)
    {
      if (i == sc_selected)
	{
	  wattron(sc_win, A_REVERSE); 
	  mvwprintw(sc_win, i+1, 1, arch[arch_selected].systems[os_selected].sc[i].ID);
	  wattroff(sc_win, A_REVERSE); 
	}
      else
	{
	  mvwprintw(sc_win, i+1, 1, arch[arch_selected].systems[os_selected].sc[i].ID);
	}
    }
  wrefresh(sc_win);
  clean_help_line(3);
  mvwprintw(help_win, 3, 2, arch[arch_selected].systems[os_selected].sc[sc_selected].desc);
  wrefresh(help_win);
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void refresh_opt(int arch_selected, int os_selected, int sc_selected, int opt_selected)
{
  int i, s, f;
  char *partial;

  werase(opt_win);
  box(opt_win, 0, 0);
  mvwprintw(opt_win, 0, 2, "Options");

  partial = (char *) malloc(SCREEN_COL - 39 - 18);

  for(i = 0; i < arch[arch_selected].systems[os_selected].sc[sc_selected].num_opt; i++)
    {
      if (i == opt_selected)
	{
	  wattron(opt_win, A_REVERSE); 
	  mvwprintw(opt_win, i+1, 1, arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].ID);
	  wattroff(opt_win, A_REVERSE); 
	}
      else
	{
	  mvwprintw(opt_win, i+1, 1, arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].ID);
	}

      if (arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].type == 0)
	{
	  s = arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].valI;
	  f = 0;  while(s != arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].sub[f].val) f++;
	  mvwprintw(opt_win, i+1, 18, arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].sub[f].desc);
	}

      if (arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].type == 1)
	{
	  snprintf(partial, SCREEN_COL - 39 - 18 - 1, "%s", arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].valC);
	  mvwprintw(opt_win, i+1, 18, partial);
	}
    }
  wrefresh(opt_win);

  clean_help_line(4);
  mvwprintw(help_win, 4, 2, arch[arch_selected].systems[os_selected].sc[sc_selected].func[opt_selected].desc);
  wrefresh(help_win);
  free(partial);
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void  refresh_sel_opt(WINDOW *win, submenu *sub, int num_items, int selected)
{
  int i;

  for(i = 0; i < num_items; i++)
    {
      if (i == selected)
	{
	  wattron(win, A_REVERSE); 
	  mvwprintw(win, i+3, 2, sub[i].desc);
	  wattroff(win, A_REVERSE); 
	}
      else
	{
	  mvwprintw(win, i+3, 2, sub[i].desc);
	}
    }
  wrefresh(win);
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void set_default(int arch_selected, int os_selected, int sc_selected)
{
  int s, i=0;

  while(arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].ID != NULL)
    {
      if (arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].type == 0)
	{
	  s = arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].defaultI;
	  arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].valI = s;
	}

      if (arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].type == 1)
	{
	  arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].valC = (char *) malloc(LEN_CMD);
	  strncpy(arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].valC, arch[arch_selected].systems[os_selected].sc[sc_selected].func[i].defaultC, LEN_CMD);
	}    
      i++;
    }
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void clean_help_line(int line)
{
  mvwprintw(help_win, line, 1, "                                                                          ");
  wrefresh(help_win);
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

char *make_SC_printable(char *code)
{
  int h, len;
  char *sc;

  len = strlen(code);
  sc = (char *) malloc(len*5);
  sprintf(sc, "\"");
  for(h = 0; h < len; h++)
    {
      sprintf(sc, "%s\\x%.2x", sc, (unsigned char) code[h]);
      if ((h+1)%16==0) 
	{
	  if (h+1 < len) sprintf(sc, "%s\"\n\"", sc);
	}
    }
  sprintf(sc, "%s\"\n", sc);
  return sc;
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void refresh_output(WINDOW *win, int selected, char *printable_sc, int real_sc_len)
{
  int line, sc_len;
  char *p0, *p1;

  werase(win);
  box(win, 0, 0);

  sc_len = strlen(printable_sc);
  p0 = p1 = printable_sc;
  line = 1;
  while (p1 < (printable_sc + sc_len))
    {
      p1 = strchr(p0, '\n');
      *p1 = '\0';
      mvwprintw(win, line++, 2, "%s", p0);
      *p1 = '\n';
      p0 = ++p1;
    }

  line += 2;

  switch(selected)
    {
    case 0:
      wattron(win, A_REVERSE); 
      mvwprintw(win, line, 2, "[Discard]");
      wattroff(win, A_REVERSE); 
      mvwprintw(win, line, 12, "[Save]");
      mvwprintw(win, line, 19, "[Try]");
      break;

    case 1:
      mvwprintw(win, line, 2, "[Discard]");
      wattron(win, A_REVERSE); 
      mvwprintw(win, line, 12, "[Save]");
      wattroff(win, A_REVERSE); 
      mvwprintw(win, line, 19, "[Try]");
      break;

    case 2:
      mvwprintw(win, line, 2, "[Discard]");
      mvwprintw(win, line, 12, "[Save]");
      wattron(win, A_REVERSE); 
      mvwprintw(win, line, 19, "[Try]");
      wattroff(win, A_REVERSE); 
      break;
    }
  mvwprintw(win, line, 58, "%d Bytes", real_sc_len);
      wrefresh(win);   
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

char *make_desc(char *string)
{
  int start = LENROW, pre = 0 ;
  char *buff, *p;

  p = buff = (char *)  malloc(strlen(string)+30);

  while(start < strlen(string))
    {
      while(string[start] != ' ')
	{
	  start--;
	}
      start++;
      memcpy(p, string+pre, start-pre);
      p +=  (start-pre);
      p[1] = 0;
      sprintf(p, "\n * \0");
      p += 4;
      pre = start;
      start += LENROW;
    }
  memcpy(p, string+pre, strlen(string+pre));
  p[strlen(string+pre)]='\0';
  return buff;
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void build_sc(architecures arch, os system, shellcodes sc)
{
  int i, c, selected, z;
  int win_len, win_row, sc_len, real_sc_len;
  char *shellcode, *printable_sc, *sc_tmp, file_name[LEN_CMD], desc[512];
  char *new_desc, type[14];
  WINDOW *output_win;
  FILE *fd = NULL;

  desc[0] = '\0';

  void (*f)();

  i = 0;
  switch(sc.type)
    {
    case 0: // write
      shellcode = generate_write_SC(arch.val, system.val, sc.func[0].valI, sc.func[1].valC, sc.func[2].valI, sc.func[3].valI, sc.func[4].valI);

      // Description
      if (sc.func[4].valI == 1) sprintf(type, " polymorphic ");
      else sprintf(type, " ");
      z = 0;  while(sc.func[0].valI != sc.func[0].sub[z].val) z++;
      sprintf(desc, " * This is a %s/%s%sShellCode that writes \"%s\" to %s", system.ID, arch.ID, type, sc.func[1].valC, sc.func[0].sub[z].desc);
      if (sc.func[2].valI > 0 && sc.func[3].valI > 0) 
	{
	  sprintf(desc, "%s after a %s and a %s", desc, sc.func[2].sub[sc.func[2].valI].desc, sc.func[3].sub[sc.func[3].valI].desc);
	}
      else
	{
	  if (sc.func[2].valI > 0) sprintf(desc, "%s after a %s", desc, sc.func[2].sub[sc.func[2].valI].desc);
	  if (sc.func[3].valI > 0) sprintf(desc, "%s after a %s", desc, sc.func[3].sub[sc.func[3].valI].desc);
	}
      sprintf(desc, "%s.\n\0", desc);
      break;

    case 1: // file write
      shellcode = generate_file_write_SC(arch.val, system.val, sc.func[0].valC, sc.func[1].valC, sc.func[2].valI, sc.func[3].valI, sc.func[4].valI, sc.func[5].valI);

      // Description
      if (sc.func[5].valI == 1) sprintf(type, " polymorphic ");
      else sprintf(type, " ");

      sprintf(desc, " * This is a %s/%s%sShellCode that ", system.ID, arch.ID, type);

      if (sc.func[2].valI > 0 && sc.func[3].valI > 0) 
	{
	  sprintf(desc, "%sexec a %s and a %s, then ", desc, sc.func[2].sub[sc.func[2].valI].desc, sc.func[3].sub[sc.func[3].valI].desc);
	}
      else
	{
	  if (sc.func[2].valI > 0) sprintf(desc, "%sexec a %s, then ", desc, sc.func[2].sub[sc.func[2].valI].desc);
	  if (sc.func[3].valI > 0) sprintf(desc, "%sexec a %s, then ", desc, sc.func[3].sub[sc.func[3].valI].desc);
	} 

      if (sc.func[4].valI == 1 && arch.val == I386 && system.val == LINUX) sprintf(desc, "%sevade from chroot and ", desc);
      
      sprintf(desc, "%swrites \"%s\" to %s", desc, sc.func[1].valC, sc.func[0].valC);
      sprintf(desc, "%s.\n\0", desc);
      break;

    case 2: // exec
      shellcode = generate_exec_SC(arch.val, system.val, sc.func[0].valC, sc.func[1].valI, sc.func[2].valI, sc.func[3].valI, sc.func[4].valI, sc.func[5].valI);

      // Description
      if (sc.func[5].valI == 1) sprintf(type, " polymorphic ");
      else sprintf(type, " ");

      sprintf(desc, " * This is a %s/%s%sShellCode that ", system.ID, arch.ID, type);

      if (sc.func[1].valI > 0 && sc.func[2].valI > 0) 
	{
	  sprintf(desc, "%sexec a %s and a %s, then ", desc, sc.func[1].sub[sc.func[1].valI].desc, sc.func[2].sub[sc.func[2].valI].desc);
	}
      else
	{
	  if (sc.func[1].valI > 0) sprintf(desc, "%sexec a %s, then ", desc, sc.func[1].sub[sc.func[1].valI].desc);
	  if (sc.func[2].valI > 0) sprintf(desc, "%sexec a %s, then ", desc, sc.func[2].sub[sc.func[2].valI].desc);
	} 

      if (sc.func[3].valI == 1 && arch.val == I386 && system.val == LINUX) sprintf(desc, "%sevade from chroot and ", desc);     
      sprintf(desc, "%sexecutes %s", desc, sc.func[0].valC);
      if (sc.func[4].valI > 0) sprintf(desc, "%s and exit(0)", desc);
      sprintf(desc, "%s.\n\0", desc);
      break;

    case 3: // bind
      shellcode = generate_bind_SC(arch.val, system.val, sc.func[0].valC, sc.func[1].valI, sc.func[2].valI, sc.func[3].valI, sc.func[4].valI, atoi(sc.func[5].valC), sc.func[6].valI, sc.func[7].valI);

      // Description
      if (sc.func[7].valI == 1) sprintf(type, " polymorphic ");
      else sprintf(type, " ");

      sprintf(desc, " * This is a %s/%s%sShellCode that", system.ID, arch.ID, type);

      if (sc.func[1].valI > 0 && sc.func[2].valI > 0) 
	{
	  sprintf(desc, "%s exec a %s and a %s, then", desc, sc.func[1].sub[sc.func[1].valI].desc, sc.func[2].sub[sc.func[2].valI].desc);
	}
      else
	{
	  if (sc.func[1].valI > 0) sprintf(desc, "%s exec a %s, then", desc, sc.func[1].sub[sc.func[1].valI].desc);
	  if (sc.func[2].valI > 0) sprintf(desc, "%s exec a %s, then", desc, sc.func[2].sub[sc.func[2].valI].desc);
	} 

      sprintf(desc, "%s listens to the port %d and when receves a connection ", desc, atoi(sc.func[5].valC));

      if (sc.func[6].valI == 1) sprintf(desc, "%sfork()s. The child ", desc);

      if (sc.func[3].valI == 1 && arch.val == I386 && system.val == LINUX) sprintf(desc, "%sfirst evade from chroot then ", desc);

      sprintf(desc, "%sexecutes %s", desc, sc.func[0].valC);
      if (sc.func[4].valI == 1) sprintf(desc, "%s and exit(0)", desc);
        
      if (sc.func[6].valI == 1) sprintf(desc, "%s. The father listens for the next connection", desc);
      sprintf(desc, "%s.\n", desc);
      break;

    case 4: // connect back
      shellcode = generate_connect_back_SC(arch.val, system.val, sc.func[0].valC, sc.func[1].valI, sc.func[2].valI, sc.func[3].valI, sc.func[4].valI, sc.func[5].valC, atoi(sc.func[6].valC), sc.func[7].valI);

      // Description
      if (sc.func[7].valI == 1) sprintf(type, " polymorphic ");
      else sprintf(type, " ");

      sprintf(desc, " * This is a %s/%s%sShellCode that opens a connection to %s:%d and", system.ID, arch.ID, type, sc.func[5].valC, atoi(sc.func[6].valC));

      if (sc.func[1].valI > 0 && sc.func[2].valI > 0) 
	{
	  sprintf(desc, "%s exec a %s and a %s, then", desc, sc.func[1].sub[sc.func[1].valI].desc, sc.func[2].sub[sc.func[2].valI].desc);
	}
      else
	{
	  if (sc.func[1].valI > 0) sprintf(desc, "%s exec a %s, then", desc, sc.func[1].sub[sc.func[1].valI].desc);
	  if (sc.func[2].valI > 0) sprintf(desc, "%s exec a %s, then", desc, sc.func[2].sub[sc.func[2].valI].desc);
	} 

      if (sc.func[3].valI == 1 && arch.val == I386 && system.val == LINUX) sprintf(desc, "%s evade from chroot and", desc);  
      sprintf(desc, "%s executes %s", desc, sc.func[0].valC);
      if (sc.func[4].valI == 1 > 0) 
	{
	  sprintf(desc, "%s and exit(0)", desc);
	}      
      sprintf(desc, "%s.\n", desc);
      break;

    case 5: // socket reuse
      shellcode = generate_socket_reuse_SC(arch.val, system.val, sc.func[0].valC, sc.func[1].valI, sc.func[2].valI, sc.func[3].valI, sc.func[4].valI, sc.func[5].valC, atoi(sc.func[6].valC), sc.func[7].valI);

      // Description
      if (sc.func[7].valI == 1) sprintf(type, " polymorphic ");
      else sprintf(type, " ");

      sprintf(desc, " * This is a %s/%s%sShellCode that reuse the opened connection from %s:%d and", system.ID, arch.ID, type, sc.func[5].valC, atoi(sc.func[6].valC));

      if (sc.func[1].valI > 0 && sc.func[2].valI > 0) 
	{
	  sprintf(desc, "%s exec a %s and a %s, then", desc, sc.func[1].sub[sc.func[1].valI].desc, sc.func[2].sub[sc.func[2].valI].desc);
	}
      else
	{
	  if (sc.func[1].valI > 0) sprintf(desc, "%s exec a %s, then", desc, sc.func[1].sub[sc.func[1].valI].desc);
	  if (sc.func[2].valI > 0) sprintf(desc, "%s exec a %s, then", desc, sc.func[2].sub[sc.func[2].valI].desc);
	} 

      if (sc.func[3].valI == 1 && arch.val == I386 && system.val == LINUX) sprintf(desc, "%s evade from chroot and", desc);  
      sprintf(desc, "%s executes %s", desc, sc.func[0].valC);
      if (sc.func[4].valI == 1 > 0) 
	{
	  sprintf(desc, "%s and exit(0)", desc);
	}      
      sprintf(desc, "%s.\n", desc);
      break;
    }

  new_desc = make_desc(desc);
  strcpy(desc, new_desc);
  free (new_desc);

  real_sc_len = strlen(shellcode);
  printable_sc = make_SC_printable(shellcode);
  sc_len = strlen(printable_sc);
  win_row = sc_len / 67;
  if ((win_row * 67) != sc_len) win_row++;

  win_len = ((SCREEN_COL - (66 + 4))/2);
  output_win = newwin(win_row + 6, 66 + 4, ((SCREEN_ROW - win_row - 6)/2), win_len);
  wbkgdset(output_win, COLOR_PAIR(POPUP_MSG_COLOR));

  selected = 0;
  while(1)
    {
      refresh_output(output_win, selected, printable_sc, real_sc_len);
      c = wgetch(arch_win);
      switch(c)
        {
	case KEY_LEFT:
	  if (selected != 0) selected--;
	  break;

	case KEY_RIGHT:
	  if (selected != 2) selected++;
	  break;
	  
	case 10: // ENTER
	  if (selected == 0)  // Discard
	    {	      
	      delwin(output_win);
	      free(printable_sc);
	      free(shellcode);
	      return;
	    }

	  if (selected == 1)  // Save
	    {
	      file_name[0] = 0;
	      read_str(file_name, "Type the file name:");
	      fd = fopen(file_name, "w");
	      if (fd != NULL)
		{
		  fd = fopen(file_name, "w");
		  fprintf(fd, header, desc);
		  sc_tmp = make_SC_printable(shellcode);
		  sc_tmp[strlen(sc_tmp) - 1] = 0; // delete last \n
		  fprintf(fd, code, sc_tmp);
		  fclose(fd);
		  free(sc_tmp);
		}
	      else
		{
		  error_msg("I can't open the file!");
		}
	    }

	  if (selected == 2)  // Try
	    {
	      endwin();
	      printf("Trying the ShellCode...\n\n"); 
	      fflush(stdout);
	      (char *) f = shellcode;
	      f();
	      return;
	    }
	  break;
	}
    }
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void about()
{
  int msg_len, c, win_len;
  WINDOW *about_win;

  msg_len = 35;
  win_len = ((SCREEN_COL - (msg_len + 4))/2);
  
  about_win = newwin(15, msg_len + 4, ((SCREEN_ROW - 15)/2), win_len);
  keypad(about_win, TRUE); // Activate all keys 
  wbkgdset(about_win, COLOR_PAIR(POPUP_MSG_COLOR));
  werase(about_win);
  mvwprintw(about_win, 1, 0, "%s", ORK);
  box(about_win, 0, 0);

  mvwprintw(about_win, 8, 3, "ShellCode Builder v. %s", SCBUILDER_VERSION);
  mvwprintw(about_win, 9, 3, "Using libShellCode v. %s", libShellCode_version());

  wattron(about_win, A_REVERSE); 
  mvwprintw(about_win, 12, (msg_len - 2) / 2, "[ OK ]");
  wrefresh(about_win);
  c = wgetch(about_win);
  delwin(about_win);
}


/*
 * Function:   
 * Parameters:  
 * Return:
 * Description:
 *
 */

void error_msg(char *message)
{
  int msg_len, c, win_len;
  WINDOW *error_win;

  msg_len = strlen (message);
  win_len = ((SCREEN_COL - (msg_len + 4))/2);
  
  // Create error window
  error_win = newwin(5, msg_len + 4, ((SCREEN_ROW - 5)/2), win_len);
  keypad(error_win, TRUE); // Activate all keys 
  wbkgdset(error_win, COLOR_PAIR(ERROR_MSG_COLOR));
  werase(error_win);
  box(error_win, 0, 0);
  mvwprintw(error_win, 1, 2, "%s", message);
  wattron(error_win, A_REVERSE); 
  mvwprintw(error_win, 3, (msg_len - 2) / 2, "[ OK ]");
  wrefresh(error_win);
  c = wgetch(error_win);
  delwin(error_win);
}
